@@ -31,16 +31,19 @@
  * are welcome.
  */
 
+#include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "libssh/priv.h"
+#include "libssh/session.h"
+#include "libssh/crypto.h"
+#include "libssh/wrapper.h"
 
 #ifdef HAVE_LIBGCRYPT
 #include <gcrypt.h>
 
-#include "libssh/crypto.h"
 
 static int alloc_key(struct crypto_struct *cipher) {
     cipher->key = malloc(cipher->keylen);
@@ -154,29 +157,31 @@ static void blowfish_decrypt(struct crypto_struct *cipher, void *in,
 }
 
 static int aes_set_key(struct crypto_struct *cipher, void *key, void *IV) {
+  int mode=GCRY_CIPHER_MODE_CBC;
   if (cipher->key == NULL) {
     if (alloc_key(cipher) < 0) {
       return -1;
     }
-
+    if(strstr(cipher->name,"-ctr"))
+      mode=GCRY_CIPHER_MODE_CTR;
     switch (cipher->keysize) {
       case 128:
         if (gcry_cipher_open(&cipher->key[0], GCRY_CIPHER_AES128,
-              GCRY_CIPHER_MODE_CBC, 0)) {
+              mode, 0)) {
           SAFE_FREE(cipher->key);
           return -1;
         }
         break;
       case 192:
         if (gcry_cipher_open(&cipher->key[0], GCRY_CIPHER_AES192,
-              GCRY_CIPHER_MODE_CBC, 0)) {
+              mode, 0)) {
           SAFE_FREE(cipher->key);
           return -1;
         }
         break;
       case 256:
         if (gcry_cipher_open(&cipher->key[0], GCRY_CIPHER_AES256,
-              GCRY_CIPHER_MODE_CBC, 0)) {
+              mode, 0)) {
           SAFE_FREE(cipher->key);
           return -1;
         }
@@ -186,9 +191,17 @@ static int aes_set_key(struct crypto_struct *cipher, void *key, void *IV) {
       SAFE_FREE(cipher->key);
       return -1;
     }
-    if (gcry_cipher_setiv(cipher->key[0], IV, 16)) {
-      SAFE_FREE(cipher->key);
-      return -1;
+    if(mode == GCRY_CIPHER_MODE_CBC){
+      if (gcry_cipher_setiv(cipher->key[0], IV, 16)) {
+
+        SAFE_FREE(cipher->key);
+        return -1;
+      }
+    } else {
+      if(gcry_cipher_setctr(cipher->key[0],IV,16)){
+        SAFE_FREE(cipher->key);
+        return -1;
+      }
     }
   }
 
@@ -262,11 +275,11 @@ static int des3_1_set_key(struct crypto_struct *cipher, void *key, void *IV) {
       SAFE_FREE(cipher->key);
       return -1;
     }
-    if (gcry_cipher_setkey(cipher->key[1], key + 8, 8)) {
+    if (gcry_cipher_setkey(cipher->key[1], (unsigned char *)key + 8, 8)) {
       SAFE_FREE(cipher->key);
       return -1;
     }
-    if (gcry_cipher_setiv(cipher->key[1], IV + 8, 8)) {
+    if (gcry_cipher_setiv(cipher->key[1], (unsigned char *)IV + 8, 8)) {
       SAFE_FREE(cipher->key);
       return -1;
     }
@@ -276,11 +289,11 @@ static int des3_1_set_key(struct crypto_struct *cipher, void *key, void *IV) {
       SAFE_FREE(cipher->key);
       return -1;
     }
-    if (gcry_cipher_setkey(cipher->key[2], key + 16, 8)) {
+    if (gcry_cipher_setkey(cipher->key[2], (unsigned char *)key + 16, 8)) {
       SAFE_FREE(cipher->key);
       return -1;
     }
-    if (gcry_cipher_setiv(cipher->key[2], IV + 16, 8)) {
+    if (gcry_cipher_setiv(cipher->key[2], (unsigned char *)IV + 16, 8)) {
       SAFE_FREE(cipher->key);
       return -1;
     }
@@ -316,6 +329,39 @@ static struct crypto_struct ssh_ciphertab[] = {
     .cbc_encrypt     = blowfish_encrypt,
     .cbc_decrypt     = blowfish_decrypt
   },
+  {
+    .name            = "aes128-ctr",
+    .blocksize       = 16,
+    .keylen          = sizeof(gcry_cipher_hd_t),
+    .key             = NULL,
+    .keysize         = 128,
+    .set_encrypt_key = aes_set_key,
+    .set_decrypt_key = aes_set_key,
+    .cbc_encrypt     = aes_encrypt,
+    .cbc_decrypt     = aes_encrypt
+  },
+  {
+      .name            = "aes192-ctr",
+      .blocksize       = 16,
+      .keylen          = sizeof(gcry_cipher_hd_t),
+      .key             = NULL,
+      .keysize         = 192,
+      .set_encrypt_key = aes_set_key,
+      .set_decrypt_key = aes_set_key,
+      .cbc_encrypt     = aes_encrypt,
+      .cbc_decrypt     = aes_encrypt
+  },
+  {
+      .name            = "aes256-ctr",
+      .blocksize       = 16,
+      .keylen          = sizeof(gcry_cipher_hd_t),
+      .key             = NULL,
+      .keysize         = 256,
+      .set_encrypt_key = aes_set_key,
+      .set_decrypt_key = aes_set_key,
+      .cbc_encrypt     = aes_encrypt,
+      .cbc_decrypt     = aes_encrypt
+  },
   {
     .name            = "aes128-cbc",
     .blocksize       = 16,
@@ -405,7 +451,7 @@ static struct crypto_struct ssh_ciphertab[] = {
 #include <openssl/des.h>
 #endif
 
-#if (OPENSSL_VERSION_NUMBER<0x009070000)
+#if (OPENSSL_VERSION_NUMBER<0x00907000L)
 #define OLD_CRYPTO
 #endif
 
@@ -567,6 +613,24 @@ static void aes_decrypt(struct crypto_struct *cipher, void *in, void *out,
     unsigned long len, void *IV) {
   AES_cbc_encrypt(in, out, len, cipher->key, IV, AES_DECRYPT);
 }
+
+/** @internal
+ * @brief encrypts/decrypts data with stream cipher AES_ctr128. 128 bits is actually
+ * the size of the CTR counter and incidentally the blocksize, but not the keysize.
+ * @param len[in] must be a multiple of AES128 block size.
+ */
+static void aes_ctr128_encrypt(struct crypto_struct *cipher, void *in, void *out,
+    unsigned long len, void *IV) {
+  unsigned char tmp_buffer[128/8];
+  unsigned int num=0;
+  /* Some things are special with ctr128 :
+   * In this case, tmp_buffer is not being used, because it is used to store temporary data
+   * when an encryption is made on lengths that are not multiple of blocksize.
+   * Same for num, which is being used to store the current offset in blocksize in CTR
+   * function.
+   */
+  AES_ctr128_encrypt(in, out, len, cipher->key, IV, tmp_buffer, &num);
+}
 #endif /* HAS_AES */
 
 #ifdef HAS_DES
@@ -577,11 +641,11 @@ static int des3_set_key(struct crypto_struct *cipher, void *key) {
     }
 
     DES_set_odd_parity(key);
-    DES_set_odd_parity(key + 8);
-    DES_set_odd_parity(key + 16);
+    DES_set_odd_parity((void*)((uint8_t*)key + 8));
+    DES_set_odd_parity((void*)((uint8_t*)key + 16));
     DES_set_key_unchecked(key, cipher->key);
-    DES_set_key_unchecked(key + 8, cipher->key + sizeof(DES_key_schedule));
-    DES_set_key_unchecked(key + 16, cipher->key + 2 * sizeof(DES_key_schedule));
+    DES_set_key_unchecked((void*)((uint8_t*)key + 8), (void*)((uint8_t*)cipher->key + sizeof(DES_key_schedule)));
+    DES_set_key_unchecked((void*)((uint8_t*)key + 16), (void*)((uint8_t*)cipher->key + 2 * sizeof(DES_key_schedule)));
   }
 
   return 0;
@@ -590,16 +654,16 @@ static int des3_set_key(struct crypto_struct *cipher, void *key) {
 static void des3_encrypt(struct crypto_struct *cipher, void *in,
     void *out, unsigned long len, void *IV) {
   DES_ede3_cbc_encrypt(in, out, len, cipher->key,
-      cipher->key + sizeof(DES_key_schedule),
-      cipher->key + 2 * sizeof(DES_key_schedule),
+      (void*)((uint8_t*)cipher->key + sizeof(DES_key_schedule)),
+      (void*)((uint8_t*)cipher->key + 2 * sizeof(DES_key_schedule)),
       IV, 1);
 }
 
 static void des3_decrypt(struct crypto_struct *cipher, void *in,
     void *out, unsigned long len, void *IV) {
   DES_ede3_cbc_encrypt(in, out, len, cipher->key,
-      cipher->key + sizeof(DES_key_schedule),
-      cipher->key + 2 * sizeof(DES_key_schedule),
+      (void*)((uint8_t*)cipher->key + sizeof(DES_key_schedule)),
+      (void*)((uint8_t*)cipher->key + 2 * sizeof(DES_key_schedule)),
       IV, 0);
 }
 
@@ -609,10 +673,10 @@ static void des3_1_encrypt(struct crypto_struct *cipher, void *in,
   ssh_print_hexa("Encrypt IV before", IV, 24);
 #endif
   DES_ncbc_encrypt(in, out, len, cipher->key, IV, 1);
-  DES_ncbc_encrypt(out, in, len, cipher->key + sizeof(DES_key_schedule),
-      IV + 8, 0);
-  DES_ncbc_encrypt(in, out, len, cipher->key + 2 * sizeof(DES_key_schedule),
-      IV + 16, 1);
+  DES_ncbc_encrypt(out, in, len, (void*)((uint8_t*)cipher->key + sizeof(DES_key_schedule)),
+      (void*)((uint8_t*)IV + 8), 0);
+  DES_ncbc_encrypt(in, out, len, (void*)((uint8_t*)cipher->key + 2 * sizeof(DES_key_schedule)),
+      (void*)((uint8_t*)IV + 16), 1);
 #ifdef DEBUG_CRYPTO
   ssh_print_hexa("Encrypt IV after", IV, 24);
 #endif
@@ -624,11 +688,11 @@ static void des3_1_decrypt(struct crypto_struct *cipher, void *in,
   ssh_print_hexa("Decrypt IV before", IV, 24);
 #endif
 
-  DES_ncbc_encrypt(in, out, len, cipher->key + 2 * sizeof(DES_key_schedule),
+  DES_ncbc_encrypt(in, out, len, (void*)((uint8_t*)cipher->key + 2 * sizeof(DES_key_schedule)),
       IV, 0);
-  DES_ncbc_encrypt(out, in, len, cipher->key + sizeof(DES_key_schedule),
-      IV + 8, 1);
-  DES_ncbc_encrypt(in, out, len, cipher->key, IV + 16, 0);
+  DES_ncbc_encrypt(out, in, len, (void*)((uint8_t*)cipher->key + sizeof(DES_key_schedule)),
+      (void*)((uint8_t*)IV + 8), 1);
+  DES_ncbc_encrypt(in, out, len, cipher->key, (void*)((uint8_t*)IV + 16), 0);
 
 #ifdef DEBUG_CRYPTO
   ssh_print_hexa("Decrypt IV after", IV, 24);
@@ -637,90 +701,128 @@ static void des3_1_decrypt(struct crypto_struct *cipher, void *in,
 
 #endif /* HAS_DES */
 
-/* the table of supported ciphers */
+/*
+ * The table of supported ciphers
+ *
+ * WARNING: If you modify crypto_struct, you must make sure the order is
+ * correct!
+ */
 static struct crypto_struct ssh_ciphertab[] = {
 #ifdef HAS_BLOWFISH
   {
-    .name            = "blowfish-cbc",
-    .blocksize       = 8,
-    .keylen          = sizeof (BF_KEY),
-    .key             = NULL,
-    .keysize         = 128,
-    .set_encrypt_key = blowfish_set_key,
-    .set_decrypt_key = blowfish_set_key,
-    .cbc_encrypt     = blowfish_encrypt,
-    .cbc_decrypt     = blowfish_decrypt
+    "blowfish-cbc",
+    8,
+    sizeof (BF_KEY),
+    NULL,
+    128,
+    blowfish_set_key,
+    blowfish_set_key,
+    blowfish_encrypt,
+    blowfish_decrypt
   },
 #endif /* HAS_BLOWFISH */
 #ifdef HAS_AES
   {
-    .name            = "aes128-cbc",
-    .blocksize       = 16,
-    .keylen          = sizeof(AES_KEY),
-    .key             = NULL,
-    .keysize         = 128,
-    .set_encrypt_key = aes_set_encrypt_key,
-    .set_decrypt_key = aes_set_decrypt_key,
-    .cbc_encrypt     = aes_encrypt,
-    .cbc_decrypt     = aes_decrypt
+    "aes128-ctr",
+    16,
+    sizeof(AES_KEY),
+    NULL,
+    128,
+    aes_set_encrypt_key,
+    aes_set_encrypt_key,
+    aes_ctr128_encrypt,
+    aes_ctr128_encrypt
   },
   {
-    .name            = "aes192-cbc",
-    .blocksize       = 16,
-    .keylen          = sizeof(AES_KEY),
-    .key             = NULL,
-    .keysize         = 192,
-    .set_encrypt_key = aes_set_encrypt_key,
-    .set_decrypt_key = aes_set_decrypt_key,
-    .cbc_encrypt     = aes_encrypt,
-    .cbc_decrypt     = aes_decrypt
+    "aes192-ctr",
+    16,
+    sizeof(AES_KEY),
+    NULL,
+    192,
+    aes_set_encrypt_key,
+    aes_set_encrypt_key,
+    aes_ctr128_encrypt,
+    aes_ctr128_encrypt
   },
   {
-    .name            = "aes256-cbc",
-    .blocksize       = 16,
-    .keylen          = sizeof(AES_KEY),
-    .key             = NULL,
-    .keysize         = 256,
-    .set_encrypt_key = aes_set_encrypt_key,
-    .set_decrypt_key = aes_set_decrypt_key,
-    .cbc_encrypt     = aes_encrypt,
-    .cbc_decrypt     = aes_decrypt
+    "aes256-ctr",
+    16,
+    sizeof(AES_KEY),
+    NULL,
+    256,
+    aes_set_encrypt_key,
+    aes_set_encrypt_key,
+    aes_ctr128_encrypt,
+    aes_ctr128_encrypt
+  },
+  {
+    "aes128-cbc",
+    16,
+    sizeof(AES_KEY),
+    NULL,
+    128,
+    aes_set_encrypt_key,
+    aes_set_decrypt_key,
+    aes_encrypt,
+    aes_decrypt
+  },
+  {
+    "aes192-cbc",
+    16,
+    sizeof(AES_KEY),
+    NULL,
+    192,
+    aes_set_encrypt_key,
+    aes_set_decrypt_key,
+    aes_encrypt,
+    aes_decrypt
+  },
+  {
+    "aes256-cbc",
+    16,
+    sizeof(AES_KEY),
+    NULL,
+    256,
+    aes_set_encrypt_key,
+    aes_set_decrypt_key,
+    aes_encrypt,
+    aes_decrypt
   },
 #endif /* HAS_AES */
 #ifdef HAS_DES
   {
-    .name            = "3des-cbc",
-    .blocksize       = 8,
-    .keylen          = sizeof(DES_key_schedule) * 3,
-    .key             = NULL,
-    .keysize         = 192,
-    .set_encrypt_key = des3_set_key,
-    .set_decrypt_key = des3_set_key,
-    .cbc_encrypt     = des3_encrypt,
-    .cbc_decrypt     = des3_decrypt
+    "3des-cbc",
+    8,
+    sizeof(DES_key_schedule) * 3,
+    NULL,
+    192,
+    des3_set_key,
+    des3_set_key,
+    des3_encrypt,
+    des3_decrypt
   },
   {
-    .name            = "3des-cbc-ssh1",
-    .blocksize       = 8,
-    .keylen          = sizeof(DES_key_schedule) * 3,
-    .key             = NULL,
-    .keysize         = 192,
-    .set_encrypt_key = des3_set_key,
-    .set_decrypt_key = des3_set_key,
-    .cbc_encrypt     = des3_1_encrypt,
-    .cbc_decrypt     = des3_1_decrypt
+    "3des-cbc-ssh1",
+    8,
+    sizeof(DES_key_schedule) * 3,
+    NULL,
+    192,
+    des3_set_key,
+    des3_set_key,
+    des3_1_encrypt,
+    des3_1_decrypt
   },
 #endif /* HAS_DES */
   {
-    .name            = NULL,
-    .blocksize       = 0,
-    .keylen          = 0,
-    .key             = NULL,
-    .keysize         = 0,
-    .set_encrypt_key = NULL,
-    .set_decrypt_key = NULL,
-    .cbc_encrypt     = NULL,
-    .cbc_decrypt     = NULL
+    NULL,
+    0,
+    0,
+    NULL,
+    0,
+    NULL,
+    NULL,
+    NULL,
+    NULL
   }
 };
 #endif /* OPENSSL_CRYPTO */
@@ -763,20 +865,18 @@ static void cipher_free(struct crypto_struct *cipher) {
   SAFE_FREE(cipher);
 }
 
-CRYPTO *crypto_new(void) {
-  CRYPTO *crypto;
+struct ssh_crypto_struct *crypto_new(void) {
+	struct ssh_crypto_struct *crypto;
 
-  crypto = malloc(sizeof(CRYPTO));
+  crypto = malloc(sizeof(struct ssh_crypto_struct));
   if (crypto == NULL) {
     return NULL;
   }
-
-  memset(crypto, 0, sizeof(CRYPTO));
-
+  ZERO_STRUCTP(crypto);
   return crypto;
 }
 
-void crypto_free(CRYPTO *crypto){
+void crypto_free(struct ssh_crypto_struct *crypto){
   if (crypto == NULL) {
     return;
   }
@@ -798,7 +898,7 @@ void crypto_free(CRYPTO *crypto){
   SAFE_FREE(crypto);
 }
 
-static int crypt_set_algorithms2(SSH_SESSION *session){
+static int crypt_set_algorithms2(ssh_session session){
   const char *wanted;
   int i = 0;
 
@@ -855,7 +955,7 @@ static int crypt_set_algorithms2(SSH_SESSION *session){
   return SSH_OK;
 }
 
-static int crypt_set_algorithms1(SSH_SESSION *session) {
+static int crypt_set_algorithms1(ssh_session session) {
   int i = 0;
 
   /* right now, we force 3des-cbc to be taken */
@@ -884,13 +984,13 @@ static int crypt_set_algorithms1(SSH_SESSION *session) {
   return SSH_OK;
 }
 
-int crypt_set_algorithms(SSH_SESSION *session) {
+int crypt_set_algorithms(ssh_session session) {
   return (session->version == 1) ? crypt_set_algorithms1(session) :
     crypt_set_algorithms2(session);
 }
 
 // TODO Obviously too much cut and paste here
-int crypt_set_algorithms_server(SSH_SESSION *session){
+int crypt_set_algorithms_server(ssh_session session){
     char *server = NULL;
     char *client = NULL;
     char *match = NULL;
